/*!
 * @file shader_data.h
 * @brief Definition of the Shader rendering class
*/

#ifndef _SHADER_DATA_H
#define _SHADER_DATA_H

#include "base_object.h"

#include "shader_types.h"
#include <vector>


// FORWARD DECLARATION



//! The ShaderData is a Class-wrapper around the OpenGL Shader Language (GLSL).
class ShaderData : public BaseObject
{
  ObjectListDeclaration(ShaderData);
public:
  ShaderData(const std::string& vertexShaderFile = "", const std::string& fragmentShaderFile = "");
  virtual ~ShaderData();

  Shaders::Uniform getUniform(const std::string& location) { return Shader::Uniform(this, location); }

  GLhandleARB getProgram() const { return this->shaderProgram; }
  GLhandleARB getVertexS() const { return this->vertexShader; }
  GLhandleARB getFragmentS() const { return this->fragmentShader; }

  void activateShader();

  void debug() const;
private:
  void bindShader(const char* name, const float* value, size_t size);
  void linkShaderProgram();
  bool readShader(const std::string& fileName, std::string& output);
  bool loadShaderProgramm(Shader::Type type, const std::string& fileName);
  void deleteProgram(Shader::Type type);

private:
  std::string            fragmentShaderFile;
  std::string            vertexShaderFile;

  GLhandleARB            shaderProgram;

  GLhandleARB            vertexShader;
  GLhandleARB            fragmentShader;
};

#endif /* _SHADER_DATA_H */
